@@ -285,9 +285,9 @@ str_coalesce(SEXP x){
 }
 
 static inline SEXP
-reconstruct(SEXP x, SEXP source, bool shallow_copy){
+rebuild(SEXP x, SEXP source, bool shallow_copy){
   typedef SEXP fn_t(SEXP, SEXP, bool);
-  static fn_t *fn = (fn_t*) R_GetCCallable("cheapr", "api_reconstruct");
+  static fn_t *fn = (fn_t*) R_GetCCallable("cheapr", "api_rebuild");
   return fn(x, source, shallow_copy);
 }
 
