@@ -13,11 +13,6 @@ void copy_warning(){
   Rf_warning("x is not a double vector and has been copied, it will not be replaced by reference.\n\tEnsure the result is assigned to an object if used in further calculations\n\te.g. `x <- set_log(x)`");
 }
 
-double round_nearest_even(double x){
-  x -= std::remainder(x, 1.0);
-  return x;
-}
-
 SEXP check_transform_altrep(SEXP x){
   if (ALTREP(x)){
     Rf_warning("Cannot update an ALTREP by reference, a copy has been made. \n\tEnsure the result is assigned to an object if used in further calculations\n\te.g. `x <- set_abs(x)`");
@@ -29,16 +24,17 @@ SEXP check_transform_altrep(SEXP x){
 
 #define CHEAPR_MATH_INT_LOOP(_fun_)                                         \
 for (R_xlen_t i = 0; i < n; ++i) {                                          \
-  p_out[i] = p_out[i] == NA_INTEGER ? p_out[i] : _fun_(p_out[i]);           \
+  p_out[i] = is_na_int(p_out[i]) ? p_out[i] : _fun_(p_out[i]);           \
 }                                                                           \
 
 #define CHEAPR_MATH_REAL_LOOP(_fun_)                                      \
 for (R_xlen_t i = 0; i < n; ++i) {                                        \
-  p_out[i] = p_out[i] != p_out[i] ? p_out[i] : _fun_(p_out[i]);           \
+  p_out[i] = is_na_dbl(p_out[i]) ? p_out[i] : _fun_(p_out[i]);           \
 }                                                                         \
 
 
 #define CHEAPR_TRUNC(x) (std::trunc(x) + 0.0)
+#define CHEAPR_SIGN(x) ((x > 0) - (x < 0))
 
 // Convert integer vector to plain double vector
 
@@ -46,9 +42,9 @@ SEXP convert_int_to_real(SEXP x){
   int *p_x = INTEGER(x);
   R_xlen_t n = Rf_xlength(x);
   SEXP out = SHIELD(new_vec(REALSXP, n));
-  double *p_out = REAL(out);
+  double* RESTRICT p_out = REAL(out);
   for (int i = 0; i < n; ++i){
-    p_out[i] = p_x[i] != NA_INTEGER ? static_cast<double>(p_x[i]) : NA_REAL;
+    p_out[i] = is_na_int(p_x[i]) ? NA_REAL : static_cast<double>(p_x[i]);
   }
   YIELD(1);
   return out;
@@ -197,7 +193,9 @@ SEXP cpp_set_exp(SEXP x){
   check_numeric(x);
   R_xlen_t n = Rf_xlength(x);
   int n_cores = n >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
-  SEXP out;
+
+  SEXP out = R_NilValue;
+
   if (!Rf_isReal(x)){
     copy_warning();
     out = SHIELD(convert_int_to_real(x));
@@ -221,7 +219,9 @@ SEXP cpp_set_sqrt(SEXP x){
   check_numeric(x);
   R_xlen_t n = Rf_xlength(x);
   int n_cores = n >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
-  SEXP out;
+
+  SEXP out = R_NilValue;
+
   if (!Rf_isReal(x)){
     copy_warning();
     out = SHIELD(convert_int_to_real(x));
@@ -244,11 +244,11 @@ SEXP cpp_set_sqrt(SEXP x){
 SEXP cpp_set_add(SEXP x, SEXP y){
   check_numeric(x);
   check_numeric(y);
-  int NP = 0;
+  int32_t NP = 0;
   SEXP out = SHIELD(check_transform_altrep(x)); ++NP;
-  R_xlen_t xn = Rf_xlength(out);
-  R_xlen_t yn = Rf_xlength(y);
-  int n_cores = xn >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
+  uint_fast64_t xn = Rf_xlength(out);
+  uint_fast64_t yn = Rf_xlength(y);
+  uint_fast64_t yi = 0;
 
   if (xn > 0){
     if (yn > xn){
@@ -269,24 +269,19 @@ SEXP cpp_set_add(SEXP x, SEXP y){
   case INTSXP: {
     int *p_x = INTEGER(out);
     int *p_y = INTEGER(y);
-#pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
-    for (R_xlen_t i = 0; i < xn; ++i) {
-      R_xlen_t yi = i % yn;
-      p_x[i] = (p_x[i] == NA_INTEGER || p_y[yi] == NA_INTEGER) ?
+    for (uint_fast64_t i = 0; i < xn; yi = (++yi == yn) ? 0 : yi, ++i){
+      p_x[i] = (is_na_int(p_x[i]) || is_na_int(p_y[yi])) ?
       NA_INTEGER : p_x[i] + p_y[yi];
     }
     break;
   }
   case REALSXP: {
     copy_warning();
-    SHIELD(out = coerce_vec(out, REALSXP));
-    ++NP;
+    SHIELD(out = coerce_vec(out, REALSXP)); ++NP;
     double *p_x = REAL(out);
     double *p_y = REAL(y);
-#pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
-    for (R_xlen_t i = 0; i < xn; ++i) {
-      R_xlen_t yi = i % yn;
-      p_x[i] = (p_x[i] != p_x[i]  || p_y[yi] != p_y[yi]) ?
+    for (uint_fast64_t i = 0; i < xn; yi = (++yi == yn) ? 0 : yi, ++i){
+      p_x[i] = (is_na_dbl(p_x[i]) || is_na_dbl(p_y[yi]))?
       NA_REAL : p_x[i] + p_y[yi];
     }
     break;
@@ -300,10 +295,8 @@ SEXP cpp_set_add(SEXP x, SEXP y){
   case INTSXP: {
     double *p_x = REAL(out);
     int *p_y = INTEGER(y);
-#pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
-    for (R_xlen_t i = 0; i < xn; ++i) {
-      R_xlen_t yi = i % yn;
-      p_x[i] = (p_x[i] !=  p_x[i] || p_y[yi] == NA_INTEGER) ?
+    for (uint_fast64_t i = 0; i < xn; yi = (++yi == yn) ? 0 : yi, ++i){
+      p_x[i] = (is_na_dbl(p_x[i]) || is_na_int(p_y[yi])) ?
       NA_REAL : p_x[i] + p_y[yi];
     }
     break;
@@ -311,11 +304,8 @@ SEXP cpp_set_add(SEXP x, SEXP y){
   case REALSXP: {
     double *p_x = REAL(out);
     double *p_y = REAL(y);
-#pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
-    for (R_xlen_t i = 0; i < xn; ++i) {
-      R_xlen_t yi = i % yn;
-      p_x[i] = (p_x[i] != p_x[i]  || p_y[yi] != p_y[yi]) ?
-      NA_REAL : p_x[i] + p_y[yi];
+    for (uint_fast64_t i = 0; i < xn; yi = (++yi == yn) ? 0 : yi, ++i){
+      p_x[i] = (is_na_dbl(p_x[i]) || is_na_dbl(p_y[yi])) ? NA_REAL : p_x[i] + p_y[yi];
     }
     break;
   }
@@ -330,12 +320,12 @@ SEXP cpp_set_add(SEXP x, SEXP y){
 SEXP cpp_set_subtract(SEXP x, SEXP y){
   check_numeric(x);
   check_numeric(y);
-  int NP = 0;
-  SEXP out = SHIELD(check_transform_altrep(x));
-  ++NP;
-  R_xlen_t xn = Rf_xlength(out);
-  R_xlen_t yn = Rf_xlength(y);
-  int n_cores = xn >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
+  int32_t NP = 0;
+  SEXP out = SHIELD(check_transform_altrep(x)); ++NP;
+  uint_fast64_t xn = Rf_xlength(out);
+  uint_fast64_t yn = Rf_xlength(y);
+  uint_fast64_t yi = 0;
+
   if (xn > 0){
     if (yn > xn){
       YIELD(NP);
@@ -346,6 +336,7 @@ SEXP cpp_set_subtract(SEXP x, SEXP y){
       Rf_error("length(y) must be be non-zero");
     }
   }
+
   switch (TYPEOF(out)){
   case LGLSXP:
   case INTSXP: {
@@ -354,24 +345,19 @@ SEXP cpp_set_subtract(SEXP x, SEXP y){
   case INTSXP: {
     int *p_x = INTEGER(out);
     int *p_y = INTEGER(y);
-#pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
-    for (R_xlen_t i = 0; i < xn; ++i) {
-      R_xlen_t yi = i % yn;
-      p_x[i] = (p_x[i] == NA_INTEGER || p_y[yi] == NA_INTEGER) ?
+    for (uint_fast64_t i = 0; i < xn; yi = (++yi == yn) ? 0 : yi, ++i){
+      p_x[i] = (is_na_int(p_x[i]) || is_na_int(p_y[yi])) ?
       NA_INTEGER : p_x[i] - p_y[yi];
     }
     break;
   }
   case REALSXP: {
     copy_warning();
-    SHIELD(out = coerce_vec(out, REALSXP));
-    ++NP;
+    SHIELD(out = coerce_vec(out, REALSXP)); ++NP;
     double *p_x = REAL(out);
     double *p_y = REAL(y);
-#pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
-    for (R_xlen_t i = 0; i < xn; ++i) {
-      R_xlen_t yi = i % yn;
-      p_x[i] = (p_x[i] != p_x[i]  || p_y[yi] != p_y[yi]) ?
+    for (uint_fast64_t i = 0; i < xn; yi = (++yi == yn) ? 0 : yi, ++i){
+      p_x[i] = (is_na_dbl(p_x[i]) || is_na_dbl(p_y[yi]))?
       NA_REAL : p_x[i] - p_y[yi];
     }
     break;
@@ -385,10 +371,8 @@ SEXP cpp_set_subtract(SEXP x, SEXP y){
   case INTSXP: {
     double *p_x = REAL(out);
     int *p_y = INTEGER(y);
-#pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
-    for (R_xlen_t i = 0; i < xn; ++i) {
-      R_xlen_t yi = i % yn;
-      p_x[i] = (p_x[i] !=  p_x[i] || p_y[yi] == NA_INTEGER) ?
+    for (uint_fast64_t i = 0; i < xn; yi = (++yi == yn) ? 0 : yi, ++i){
+      p_x[i] = (is_na_dbl(p_x[i]) || is_na_int(p_y[yi])) ?
       NA_REAL : p_x[i] - p_y[yi];
     }
     break;
@@ -396,11 +380,8 @@ SEXP cpp_set_subtract(SEXP x, SEXP y){
   case REALSXP: {
     double *p_x = REAL(out);
     double *p_y = REAL(y);
-#pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
-    for (R_xlen_t i = 0; i < xn; ++i) {
-      R_xlen_t yi = i % yn;
-      p_x[i] = (p_x[i] != p_x[i]  || p_y[yi] != p_y[yi]) ?
-      NA_REAL : p_x[i] - p_y[yi];
+    for (uint_fast64_t i = 0; i < xn; yi = (++yi == yn) ? 0 : yi, ++i){
+      p_x[i] = (is_na_dbl(p_x[i]) || is_na_dbl(p_y[yi])) ? NA_REAL : p_x[i] - p_y[yi];
     }
     break;
   }
@@ -415,12 +396,12 @@ SEXP cpp_set_subtract(SEXP x, SEXP y){
 SEXP cpp_set_multiply(SEXP x, SEXP y){
   check_numeric(x);
   check_numeric(y);
-  int NP = 0;
-  SEXP out = SHIELD(check_transform_altrep(x));
-  ++NP;
-  R_xlen_t xn = Rf_xlength(out);
-  R_xlen_t yn = Rf_xlength(y);
-  int n_cores = xn >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
+  int32_t NP = 0;
+  SEXP out = SHIELD(check_transform_altrep(x)); ++NP;
+  uint_fast64_t xn = Rf_xlength(out);
+  uint_fast64_t yn = Rf_xlength(y);
+  uint_fast64_t yi = 0;
+
   if (xn > 0){
     if (yn > xn){
       YIELD(NP);
@@ -439,9 +420,8 @@ SEXP cpp_set_multiply(SEXP x, SEXP y){
   case INTSXP: {
     int *p_x = INTEGER(out);
     int *p_y = INTEGER(y);
-#pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
-    for (R_xlen_t i = 0; i < xn; ++i) {
-      R_xlen_t yi = i % yn;
+
+    for (uint_fast64_t i = 0; i < xn; yi = (++yi == yn) ? 0 : yi, ++i){
       p_x[i] = (p_x[i] == NA_INTEGER || p_y[yi] == NA_INTEGER) ?
       NA_INTEGER : p_x[i] * p_y[yi];
     }
@@ -449,14 +429,11 @@ SEXP cpp_set_multiply(SEXP x, SEXP y){
   }
   case REALSXP: {
     copy_warning();
-    SHIELD(out = coerce_vec(out, REALSXP));
-    ++NP;
+    SHIELD(out = coerce_vec(out, REALSXP)); ++NP;
     double *p_x = REAL(out);
     double *p_y = REAL(y);
-#pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
-    for (R_xlen_t i = 0; i < xn; ++i) {
-      R_xlen_t yi = i % yn;
-      p_x[i] = (p_x[i] != p_x[i]  || p_y[yi] != p_y[yi]) ?
+    for (uint_fast64_t i = 0; i < xn; yi = (++yi == yn) ? 0 : yi, ++i){
+      p_x[i] = (is_na_dbl(p_x[i]) || is_na_dbl(p_y[yi])) ?
       NA_REAL : p_x[i] * p_y[yi];
     }
     break;
@@ -468,14 +445,10 @@ SEXP cpp_set_multiply(SEXP x, SEXP y){
     switch (TYPEOF(y)){
   case LGLSXP:
   case INTSXP: {
-    // SHIELD(y = coerce_vec(y, REALSXP));
-    // ++NP;
     double *p_x = REAL(out);
     int *p_y = INTEGER(y);
-#pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
-    for (R_xlen_t i = 0; i < xn; ++i) {
-      R_xlen_t yi = i % yn;
-      p_x[i] = (p_x[i] !=  p_x[i] || p_y[yi] == NA_INTEGER) ?
+    for (uint_fast64_t i = 0; i < xn; yi = (++yi == yn) ? 0 : yi, ++i){
+      p_x[i] = (is_na_dbl(p_x[i]) || is_na_int(p_y[yi])) ?
       NA_REAL : p_x[i] * p_y[yi];
     }
     break;
@@ -483,10 +456,8 @@ SEXP cpp_set_multiply(SEXP x, SEXP y){
   case REALSXP: {
     double *p_x = REAL(out);
     double *p_y = REAL(y);
-#pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
-    for (R_xlen_t i = 0; i < xn; ++i) {
-      R_xlen_t yi = i % yn;
-      p_x[i] = (p_x[i] != p_x[i]  || p_y[yi] != p_y[yi]) ?
+    for (uint_fast64_t i = 0; i < xn; yi = (++yi == yn) ? 0 : yi, ++i){
+      p_x[i] = (is_na_dbl(p_x[i]) ||is_na_dbl(p_y[yi])) ?
       NA_REAL : p_x[i] * p_y[yi];
     }
     break;
@@ -502,9 +473,10 @@ SEXP cpp_set_multiply(SEXP x, SEXP y){
 SEXP cpp_set_divide(SEXP x, SEXP y){
   check_numeric(x);
   check_numeric(y);
-  R_xlen_t xn = Rf_xlength(x);
-  R_xlen_t yn = Rf_xlength(y);
-  int n_cores = xn >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
+  uint_fast64_t xn = Rf_xlength(x);
+  uint_fast64_t yn = Rf_xlength(y);
+  uint_fast64_t yi = 0;
+
   if (xn > 0){
     if (yn > xn){
       Rf_error("length(y) must be <= length(x)");
@@ -513,7 +485,9 @@ SEXP cpp_set_divide(SEXP x, SEXP y){
       Rf_error("length(y) must be be non-zero");
     }
   }
-  SEXP out;
+
+  SEXP out = R_NilValue;
+
   if (!Rf_isReal(x)){
     copy_warning();
     out = SHIELD(convert_int_to_real(x));
@@ -525,22 +499,16 @@ SEXP cpp_set_divide(SEXP x, SEXP y){
   case INTSXP: {
     double *p_x = REAL(out);
     int *p_y = INTEGER(y);
-#pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
-    for (R_xlen_t i = 0; i < xn; ++i) {
-      R_xlen_t yi = i % yn;
-      p_x[i] = (p_x[i] !=  p_x[i] || p_y[yi] == NA_INTEGER) ?
-      NA_REAL : p_x[i] / p_y[yi];
+    for (uint_fast64_t i = 0; i < xn; yi = (++yi == yn) ? 0 : yi, ++i){
+      p_x[i] = (is_na_dbl(p_x[i]) || is_na_int(p_y[yi])) ? NA_REAL : p_x[i] / p_y[yi];
     }
     break;
   }
   case REALSXP: {
     double *p_x = REAL(out);
     double *p_y = REAL(y);
-#pragma omp parallel for simd num_threads(n_cores) if (n_cores > 1)
-    for (R_xlen_t i = 0; i < xn; ++i) {
-      R_xlen_t yi = i % yn;
-      p_x[i] = (p_x[i] != p_x[i]  || p_y[yi] != p_y[yi]) ?
-      NA_REAL : p_x[i] / p_y[yi];
+    for (uint_fast64_t i = 0; i < xn; yi = (++yi == yn) ? 0 : yi, ++i){
+      p_x[i] = (is_na_dbl(p_x[i]) || is_na_dbl(p_y[yi])) ? NA_REAL : p_x[i] / p_y[yi];
     }
     break;
   }
@@ -655,31 +623,34 @@ SEXP cpp_set_round(SEXP x, SEXP digits){
   check_numeric(x);
   check_numeric(digits);
   SEXP out = SHIELD(check_transform_altrep(x));
-  R_xlen_t xn = Rf_xlength(out);
-  R_xlen_t digitsn = Rf_xlength(digits);
+  uint_fast64_t xn = Rf_xlength(out);
+  uint_fast64_t digitsn = Rf_xlength(digits);
+  uint_fast64_t digitsi = 0;
+
+  double tempx, mfactor;
+
   if (xn > 0){
     if (digitsn > xn){
-      Rf_error("length(digits) must be <= length(x)");
+      YIELD(1);
+      Rf_error("`length(digits)` must be `<= length(x)`");
     }
     if (digitsn == 0){
-      Rf_error("length(digits) must be be non-zero");
+      YIELD(1);
+      Rf_error("`length(digits)` must be be non-zero");
     }
   }
-  int n_cores = xn >= CHEAPR_OMP_THRESHOLD ? num_cores() : 1;
+
   // We don't need to round integers.
+
   if (Rf_isReal(out)){
     switch (TYPEOF(digits)){
     case INTSXP: {
       double *p_x = REAL(out);
-      int *p_digits = INTEGER(digits);
-      if (n_cores > 1){
-        OMP_PARALLEL_FOR_SIMD
-        for (R_xlen_t i = 0; i < xn; ++i) {
-          R_xlen_t digitsi = i % digitsn;
-          if ( (p_x[i] == p_x[i] && p_digits[digitsi] != NA_INTEGER) ){
-            double tempx = p_x[i];
-            int tempdig = p_digits[digitsi];
-            double mfactor = std::pow(10, tempdig);
+      const int *p_digits = INTEGER(digits);
+        for (uint_fast64_t i = 0; i < xn; digitsi = (++digitsi == digitsn) ? 0 : digitsi, ++i) {
+          if ( (!is_na_dbl(p_x[i]) && !is_na_int(p_digits[digitsi])) ){
+            tempx = p_x[i];
+            mfactor = std::pow(10, p_digits[digitsi]);
             tempx *= mfactor;
             tempx = round_nearest_even(tempx);
             tempx /= mfactor;
@@ -688,52 +659,15 @@ SEXP cpp_set_round(SEXP x, SEXP digits){
             p_x[i] = NA_REAL;
           }
         }
-      } else {
-        OMP_FOR_SIMD
-        for (R_xlen_t i = 0; i < xn; ++i) {
-          R_xlen_t digitsi = i % digitsn;
-          if ( (p_x[i] == p_x[i] && p_digits[digitsi] != NA_INTEGER) ){
-            double tempx = p_x[i];
-            int tempdig = p_digits[digitsi];
-            double mfactor = std::pow(10, tempdig);
-            tempx *= mfactor;
-            tempx = round_nearest_even(tempx);
-            tempx /= mfactor;
-            p_x[i] = tempx;
-          } else {
-            p_x[i] = NA_REAL;
-          }
-        }
-      }
       break;
     }
     default: {
       double *p_x = REAL(out);
-      double *p_digits = REAL(digits);
-      if (n_cores > 1){
-        OMP_PARALLEL_FOR_SIMD
-        for (R_xlen_t i = 0; i < xn; ++i) {
-          R_xlen_t digitsi = i % digitsn;
-          if ( (p_x[i] == p_x[i] && p_digits[digitsi] == p_digits[digitsi]) ){
-            double tempx = p_x[i];
-            int tempdig = p_digits[digitsi];
-            double mfactor = std::pow(10, tempdig);
-            tempx *= mfactor;
-            tempx = round_nearest_even(tempx);
-            tempx /= mfactor;
-            p_x[i] = tempx;
-          } else {
-            p_x[i] = NA_REAL;
-          }
-        }
-      } else {
-        OMP_FOR_SIMD
-        for (R_xlen_t i = 0; i < xn; ++i) {
-          R_xlen_t digitsi = i % digitsn;
-          if ( (p_x[i] == p_x[i] && p_digits[digitsi] == p_digits[digitsi]) ){
-            double tempx = p_x[i];
-            int tempdig = p_digits[digitsi];
-            double mfactor = std::pow(10, tempdig);
+      const double *p_digits = REAL(digits);
+      for (uint_fast64_t i = 0; i < xn; digitsi = (++digitsi == digitsn) ? 0 : digitsi, ++i) {
+          if ( (!is_na_dbl(p_x[i]) && !is_na_dbl(p_digits[digitsi])) ){
+            tempx = p_x[i];
+            mfactor = std::pow(10, p_digits[digitsi]);
             tempx *= mfactor;
             tempx = round_nearest_even(tempx);
             tempx /= mfactor;
@@ -743,7 +677,6 @@ SEXP cpp_set_round(SEXP x, SEXP digits){
           }
         }
         break;
-      }
     }
     }
   }
@@ -754,10 +687,9 @@ SEXP cpp_set_round(SEXP x, SEXP digits){
 [[cpp11::register]]
 SEXP cpp_int_sign(SEXP x){
   check_numeric(x);
-  R_xlen_t n = Rf_xlength(x);
+  uint_fast64_t n = Rf_xlength(x);
   SEXP out = SHIELD(new_vec(INTSXP, n));
   int* RESTRICT p_out = INTEGER(out);
-  int res;
   switch (TYPEOF(x)){
   case LGLSXP: {
     const int *p_x = LOGICAL(x);
@@ -767,18 +699,16 @@ SEXP cpp_int_sign(SEXP x){
   case INTSXP: {
     const int *p_x = INTEGER(x);
     OMP_FOR_SIMD
-    for (R_xlen_t i = 0; i < n; ++i) {
-      res = p_x[i] == NA_INTEGER ? NA_INTEGER : (p_x[i] > 0) - (p_x[i] < 0);
-      p_out[i] = res;
+    for (uint_fast64_t i = 0; i < n; ++i) {
+      p_out[i] = is_na_int(p_x[i]) ? NA_INTEGER : static_cast<int>(CHEAPR_SIGN(p_x[i]));
     }
     break;
   }
   case REALSXP: {
     double *p_x = REAL(x);
     OMP_FOR_SIMD
-    for (R_xlen_t i = 0; i < n; ++i) {
-      res = p_x[i] != p_x[i] ? NA_INTEGER : (p_x[i] > 0) - (p_x[i] < 0);
-      p_out[i] = res;
+    for (uint_fast64_t i = 0; i < n; ++i) {
+      p_out[i] = is_na_dbl(p_x[i]) ? NA_INTEGER : static_cast<int>(CHEAPR_SIGN(p_x[i]));
     }
     break;
   }
